﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Common;
using System.Data.Objects;
using System.Data.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using MapPoint;
using Microsoft.SqlServer;
using Microsoft.SqlServer.Types;

namespace AlsBeef
{
    public partial class Map : Form
    {
        CorporateEntities db;
        List<Location> locationList;
        List<vLocation> vlocationList;
        int curLoc = -1;  // <0 == not set
        MapPoint.Map objMap;
        MapPoint.Symbol objSymb;

        public Map()
        {
            InitializeComponent();
            CreateNewMapObject();
        }

        private void CreateNewMapObject()
        {
            MPctrl.NewMap(GeoMapRegion.geoMapNorthAmerica);
            object barObj = "advanced";
            MPctrl.Toolbars.get_Item(ref barObj).Visible = true;
            MPctrl.Toolbars.LargeToolbarButtons = false;
            objMap = MPctrl.ActiveMap;
            // make sure all points of interest are turned off
            objMap.PlaceCategories.Visible = MapPoint.GeoTriState.geoFalse;
        }

        private void Map_Load(object sender, EventArgs e)
        {
            db = new CorporateEntities();
            locationList = new List<Location>();
            vlocationList = new List<vLocation>();

            ObjectQuery<Location> locationQuery = db.Locations;
            ObjectQuery<vLocation> vlocationQuery = db.vLocations;

            locationList = locationQuery.ToList();
            vlocationList = vlocationQuery.ToList();
            
            InitMapSymb();
            InitMapPins();
            SetLocation(0);
            IncDecLocation();
        }

        private void InitMapSymb()
        {
            objSymb = objMap.Symbols.Add(System.Windows.Forms.Application.StartupPath + "\\..\\..\\alsbeeflogo.bmp");
        }

        private void InitMapPins()
        {
            MapPoint.Pushpin objPin = null;
            for (int i = 0; i < locationList.Count; i++)
            {
                MapPoint.Location objLoc = objMap.GetLocation(vlocationList[i].Latitude.Value, vlocationList[i].Longitude.Value);
                objPin = objMap.AddPushpin(objLoc, locationList[i].Name);
                objPin.Symbol = objSymb.ID;   
            }
        }

        private void IncDecLocation(int cnt = 0)
        {
            // Apply the increment/decrement, and loop around if necessary (modulo)
            int newLoc = (curLoc + cnt + locationList.Count) % locationList.Count;
            SetLocation(newLoc);
        }

        private void SetLocation(int newLoc)
        {
            MapPoint.Pushpin objPin = null;

            // unhighlight previous pushpin (if we have one)
            if (curLoc >= 0)
            {
                objPin = (MapPoint.Pushpin)objMap.FindPushpin(locationList[curLoc].Name);
                objPin.Highlight = false;
            }

            // clear all previous shapes
            while (objMap.Shapes.Count > 0)
            {
                objMap.Shapes[1].Delete();
            }

            // Set this location
            curLoc = Math.Min( Math.Max(newLoc,0), locationList.Count-1);
            
            objPin = (MapPoint.Pushpin)objMap.FindPushpin(locationList[curLoc].Name);
            objMap.Location = objPin.Location;


            // draw trade area
            if (checkBox1.Checked == false)
            {
                RenderPolygon(vlocationList[curLoc].TradeAreaWKT);
            }
            else
            {
                //need to add C:\Program Files\Microsoft SQL Server\100\SDK\Assemblies\Microsoft.SqlServer.Types.dll to references
                SqlGeography TradeAreaGeog = new SqlGeography();
                using (var stream = new System.IO.MemoryStream(vlocationList[curLoc].TradeArea))
                {
                    using (var rdr = new System.IO.BinaryReader(stream))
                    {
                        TradeAreaGeog.Read(rdr);
                    }
                }
                SqlGeometry TAConvexHullGeom = new SqlGeometry();
                TAConvexHullGeom = SqlGeometry.STPolyFromText(TradeAreaGeog.STAsText(), 4326);
                TAConvexHullGeom = TAConvexHullGeom.STConvexHull();

                RenderPolygon(TradeAreaGeog.ToString(), 3355443, 0); // Gray80
                RenderPolygon(TAConvexHullGeom.ToString());
            }

            // reset zoom level
            objMap.Altitude = 30;
            objPin.Highlight = true;

            Double distance;
            distance = NearestLocation(curLoc) * 0.000621371192; //convert to miles

            label1.Text = "ID: " + locationList[curLoc].LocID.ToString();
            label2.Text = locationList[curLoc].Name + " - " + locationList[curLoc].Format;
            label3.Text = locationList[curLoc].Address + ", " + locationList[curLoc].City + ", " + locationList[curLoc].State;
            label4.Text = "Distance to next closest store: " + String.Format("{0:#,0.0}", distance) + " miles";

         }

        private void RenderPolygon(string polystring, int forecolor = 128, int weight = 3)
        {
            polystring = polystring.Replace("POLYGON ((", "");
            polystring = polystring.Replace("))", "");
            string[] stringList = polystring.Split(',');
            MapPoint.Location[] objLoc = new MapPoint.Location[stringList.Count()];
            for (int i = 0; i < stringList.Count(); i++)
            {
                string[] coords = stringList[i].Trim().Split(' ');
                objLoc[i] = objMap.GetLocation(Convert.ToDouble(coords[1]), Convert.ToDouble(coords[0]), 0);
            }
            MapPoint.Shape objShape;
            objShape = objMap.Shapes.AddPolyline(objLoc);
            objShape.Line.ForeColor = forecolor;
            objShape.Line.Weight = weight;
        }

        private double NearestLocation(int curLoc)
        {

            SqlGeography AllLocations = new SqlGeography();
            SqlGeography CurLocation = new SqlGeography();
            for (int i = 0; i < locationList.Count; i++)
            {
                SqlGeography TempLocation = new SqlGeography();
                using (var stream = new System.IO.MemoryStream(vlocationList[i].Location))
                {
                    using (var rdr = new System.IO.BinaryReader(stream))
                    {
                        TempLocation.Read(rdr);
                    }
                }
                if (i == curLoc)
                {
                    CurLocation = TempLocation;
                }
                else
                {
                    AllLocations = AllLocations.STUnion(TempLocation);
                }
            }
            return (Double)AllLocations.STDistance(CurLocation); //meters
        }

        private void prev_Click(object sender, EventArgs e)
        {
            IncDecLocation(-1);
        }

        private void next_Click(object sender, EventArgs e)
        {
            IncDecLocation(1);
        }

        private void Map_FormClosing(object sender, FormClosingEventArgs e)
        {
            db.Dispose();
            MPctrl.ActiveMap.Saved = true;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            IncDecLocation();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // clear all previous shapes
            while(objMap.Shapes.Count > 0)
            {
                objMap.Shapes[1].Delete();
            }
            MPctrl.MousePointer = MapPoint.GeoPointer.geoPointerCrosshair;
        }


        private void MPctrl_BeforeClick(object sender, AxMapPoint._IMappointCtrlEvents_BeforeClickEvent e)
        {
            if (MPctrl.MousePointer == MapPoint.GeoPointer.geoPointerCrosshair)
            {
                MPctrl.MousePointer = MapPoint.GeoPointer.geoPointerDefault;
                MapPoint.Location objLoc = objMap.XYToLocation(e.x, e.y);
                MapPoint.Pushpin objPP = objMap.AddPushpin(objLoc, "New Al's Beef Location");
                MapPoint.Shape objShape = objMap.Shapes.AddDrivetimeZone(objLoc, 3 * MapPoint.GeoTimeConstants.geoOneMinute);
                object[] vertices = objShape.Vertices as object[];
                string PolyWKT = "POLYGON((";
                bool first = true;
                string closingpoint = "";
                foreach(object vertex in vertices)
                {
                    MapPoint.Location loc = vertex as MapPoint.Location;
                    if(first){
                        closingpoint = loc.Longitude + " " + loc.Latitude;
                        first = false;
                    }
                    PolyWKT = PolyWKT + loc.Longitude + " " + loc.Latitude + ", ";
                }
                PolyWKT = PolyWKT + closingpoint + "))";

                object[] parameters =
                {   
                    new SqlParameter("Latitude",objLoc.Latitude),
                    new SqlParameter("Longitude",objLoc.Longitude),
                    new SqlParameter("PolyWKT",PolyWKT)
                };

                var retValue = db.uspAddLocation(objLoc.Longitude, objLoc.Latitude, PolyWKT);

                // requery and re-initialize map
                ObjectQuery<Location> locationQuery = db.Locations;
                ObjectQuery<vLocation> vlocationQuery = db.vLocations;
                locationList = locationQuery.ToList();
                vlocationList = vlocationQuery.ToList();
                objMap.Saved = true;
                CreateNewMapObject();
                InitMapSymb();
                InitMapPins();
                SetLocation(locationList.Count - 1);
                e.cancel = true;
            }
           
        }

        private void MPctrl_SelectionChange(object sender, AxMapPoint._IMappointCtrlEvents_SelectionChangeEvent e)
        {
            // Has the user just selected a pushpin?
            if (e.pNewSelection is MapPoint.Pushpin)
            {
                MapPoint.Pushpin ppin = e.pNewSelection as MapPoint.Pushpin;

                // Find the corresponding location object, and select it
                for (int iloc = 0; iloc < locationList.Count; iloc++)
                {
                    if (locationList[iloc].Name == ppin.Name)
                    {   // found it: select, and move to it)
                        SetLocation(iloc);
                        break;
                    }
                }
            }
        }
    }
}

